from cumin.widgets import *
from cumin.parameters import *
from cumin.objectframe import ObjectView
from hadoop import HadoopObject

class HadoopObjectFrame(CuminFrame):
    '''
    This frame houses the view page for a hadoop object
    '''
    def __init__(self, app, name, module):
        super(HadoopObjectFrame, self).__init__(app, name)

        self.module = module

        # This will hold the id of the object that should be assigned
        # to self.object. The id and the object are typically determined during
        # the "process" pass before a page is rendered.
        self.id = StringParameter(app, "id")
        self.add_parameter(self.id)

        self.host = StringParameter(app, "host")
        self.add_parameter(self.host)
        
        self.ipc = StringParameter(app, "ipc")
        self.add_parameter(self.ipc)

        # This will be given a value during the "process" pass after
        # self.id is determined (lookup by id)
        self.object = Attribute(app, "object")
        self.add_attribute(self.object)
        self.view = HadoopObjectView(app, "view", self.object)
        self.add_child(self.view)
        self.icon_href = "resource?name=action-36.png"
        self.tasks = list()

    def init(self):
        super(HadoopObjectFrame, self).init()
        assert self
        for task in self.tasks:
            task.init()

    def get_href(self, session, id, host, ipc=None):
        branch = session.branch()
        self.id.set(branch, id)
        if ipc:
            self.ipc.set(branch, ipc)
        self.host.set(branch, host)
        self.view.show(branch)
        return branch.marshal()

    def do_process(self, session):
        id = self.id.get(session)
        ipc = self.ipc.get(session)
        assert id
        obj = self.get_object(session, id, ipc)
        self.object.set(session, obj)
        super(HadoopObjectFrame, self).do_process(session)

class HadoopObjectView(ObjectView):
    def render_title(self, session):
        obj = self.object.get(session)
        if obj is not None:
            return "%s" % xml_escape(obj.Id)
        else:
            return ""

    def add_details_tab(self):
        pass
