from wooly import *
from wooly.widgets import *
from wooly.resources import *

from cumin.module import CuminModule
from cumin.widgets import *
from cumin.util import *

from namenode import NameNodeFrame
from datanode import DataNodeFrame
from jobtracker import JobTrackerFrame

from namenode import NameNodeSelector
from datanode import DataNodeSelector
from jobtracker import JobTrackerSelector
from tasktracker import TaskTrackerSelector
from scheduler import HadoopSchedulerSelector, HadoopSchedulerFrame

strings = StringCatalog(__file__)

class Module(CuminModule):
    def __init__(self, app, name):
        super(Module, self).__init__(app, name)

    def init(self):
        super(Module, self).init()

        self.frame = GridHadoopFrame(self.app, "gridhadoop", self)
        self.frame.cumin_module = self.name

        self.app.main_page.main.gridhadoop = self.frame
        self.app.main_page.main.add_tab(self.frame)


class GridHadoopFrame(CuminFrame):
    def __init__(self, app, name, module, check_owner=False):
        super(GridHadoopFrame, self).__init__(app, name)

        self.module = module

        self.view = GridHadoopView(app, name + ".view", check_owner)
        self.add_mode(self.view)

        self.name_node_frame = NameNodeFrame(app, name + ".namenodeframe", self.module)
        self.add_mode(self.name_node_frame)
        
        self.job_tracker_frame = JobTrackerFrame(app, name + ".jobtrackerframe", self.module)
        self.add_mode(self.job_tracker_frame)

    def render_title(self, session):
        return "Hadoop"

class GridHadoopView(Widget):
    def __init__(self, app, name, check_owner):
        super(GridHadoopView, self).__init__(app, name)

        self.tabs = TabbedModeSet(app, "childtabs")
        self.add_child(self.tabs)

        self.tabs.add_tab(NameNodeSelector(app, name + ".namenodes", self, check_owner))
        self.tabs.add_tab(JobTrackerSelector(app, name + ".jobtrackers", self, check_owner))      
