import logging
from cumin.module import CuminModule
from wooly.util import StringCatalog

strings = StringCatalog(__file__)

from cumin.user.widgets import *

class Module(CuminModule):
    def __init__(self, app, name):
        super(Module, self).__init__(app, name)

    def init(self):
        super(Module, self).init()

        # This module (user.grid) requires that the "user" module has been 
        # imported and initialized with a call to user.Module()
        # CuminModule classes will add themselves to "app" as an attribute
        # during intialization so we can use that to check...
        if hasattr(self.app, "user") and self.app.user:
            user_attr = self.app.user.get_user_attribute()

            o = OverviewFrame(self.app, "overview", user_attr)
            o.cumin_module = self.name
            self.app.user.add_to_view(o)

            s = SubmissionsFrame(self.app, "submissions", user_attr)
            s.cumin_module = self.name
            self.app.user.add_to_view(s)

            # The explict setting of frame path values makes this necessary
            # (search for "main.submissions.submission")
            # There has to be a way to set those paths programmatically
            self.app.user.page.main.submissions = s

        else:
            msg = "Module %s requires that the 'user' module be imported and"\
                      " initialized." % self.name
            log.error(msg)
            raise Exception(msg)

            
