from work import Work
from util import now_minus_duration
from datetime import datetime
from mint.update import AviarySubmissionUpdate

def max(stuff):
    d = 0
    for i in stuff:
        if i.qdate > d:
            d = i.qdate
    return d

def min(stuff):
    d = None
    for i in stuff:
        if d is None or i.qdate < d:
            d = i.qdate
    return d

class QuerySubmissionsAfter(Work):
    def __init__(self, ep):
        super(QuerySubmissionsAfter, self).__init__()
        self.ep = ep

    def do_the_job(self, foreman):
        ids = self.ep.app.aviary_ops.get_submission_ids_by_qdate(
            self.ep.url, 
            self.ep.size, 
            self.ep.after_index, 
            "AFTER")

        # print ("querying after", self.ep.after_index, datetime.fromtimestamp(self.ep.after_index))
        #print (ids.status, ids.got_data)

        # if data.ids is empty (remaining == 0)
        # then we should reset qdate to some point in
        # the past since we don't have an event model
        # yet.  This is how we will accomplish polling,
        # just looping over the data continuously.
        if ids.status == 0 and ids.got_data:
            # print len(ids.data.ids)
            #print ids.data.ids

            # ids.data.ids is a list of SubmissionIDs
            # Each of these ids needs to be polled to
            # get info out.
            if ids.data.ids:
                # print len(ids.data.ids)
                # print ids.data.ids
                work = PollSubmissionIDs(ids.data.ids, self.ep)
                foreman.add_work((work,))

            # ids.data.remaining tells us whether or
            # not there are any items left to get.
            # The qdate of the last item gives us the
            # starting index of the next query if there are
            # any left, otherwise we reset to a point in the
            # and start over (to provide our polling functionality)
            if ids.data.remaining:
                # Suppose it could be the case that Aviary gives
                # back an empty list but sets remaining...
                # In that case just leave after_index as is
                if ids.data.ids:
                    self.ep.after_index = ids.data.ids[-1].qdate
#                    self.ep.after_index = max(ids.data.ids)
            else:
                #print "query after reset"
                self.ep.after_index = now_minus_duration(self.ep.poll_delta)

#        else:
#            print "query after failed"

    def completed(self):
        return self.ep.abandoned or self.ep.after_queries_completed

class QuerySubmissionsBefore(Work):
    def __init__(self, ep):
        super(QuerySubmissionsBefore, self).__init__()
        self.ep = ep

    def do_the_job(self, foreman):

        # this should mark self.ep.after_queries_completed
        # or self.ep.before_queries_completed as necessary
        ids = self.ep.app.aviary_ops.get_submission_ids_by_qdate(
            self.ep.url, 
            self.ep.size, 
            self.ep.before_index, 
            "BEFORE")

        #print ("querying before", self.ep.before_index, datetime.fromtimestamp(self.ep.before_index))
        #print (ids.status, ids.got_data)

        # if data.ids is empty (remaining == 0)
        # then we should reset qdate to some point in
        # the past since we don't have an event model
        # yet.  This is how we will accomplish polling
        # We need a date to qdate function.
        if ids.status == 0 and ids.got_data:
            # print len(ids.data.ids)
            # ids.data.ids is a list of SubmissionIDs
            # each of these ids needs to be polled to
            # get info out.
            if ids.data.ids:
                work = PollSubmissionIDs(ids.data.ids, self.ep)
                foreman.add_work((work,))

            # ids.data.remaining tells us whether or
            # not there are any items after this query
            
            # the qdate of the last item gives us the
            # starting index of the next query
            if ids.data.remaining:
                self.ep.before_index = ids.data.ids[-1].qdate
#                self.ep.before_index = min(ids.data.ids)

                if self.ep.before_index <= self.ep.before_cutoff:
                    #print ("hit before cutoff, finishing",
                    #       datetime.fromtimestamp(self.ep.before_index),
                    #       datetime.fromtimestamp(self.ep.before_cutoff))
                    self.ep.before_queries_completed = True

#            else:
#                print "query before exiting"
#                self.ep.before_queries_completed =  True

#        else:
#            print "query before failed"

    def completed(self):
        return self.ep.abandoned or self.ep.before_queries_completed
               
class PollSubmissionIDs(Work):
    def __init__(self, ids, ep):
        super(PollSubmissionIDs, self).__init__()
        self.ids = ids
        self.ep = ep

    def do_the_job(self, foreman):
        # Adapt this.  AviaryOperations
        # is expecting field names beginning with
        # upper case.
        for id in self.ids:
            id.Name = id.name
            id.Owner = id.owner

        summaries = self.ep.app.aviary_ops.get_submission_summaries(
            self.ids, 
            self.ep.url, 
            include_job_summaries=False)

        # Okay, now the summaries need to be passed through the update queue to 
        # be added to the Postgres database
        # summaries can include job summaries, but we don't need those right now.
        # We need to adapt these summaries to a QMF style submission

        if summaries.status == 0 and summaries.got_data:
            for summary in summaries.data:
                summary.Name = str(summary.id.name)
                summary.Owner = str(summary.id.owner)
                summary.QDate = summary.id.qdate

                summary.Idle = summary.idle
                summary.Running = summary.running
                summary.Removed = summary.removed
                summary.Completed = summary.completed
                summary.Held = summary.held
                summary.TransferringOutput = summary.transferring_output
                summary.Suspended = summary.suspended

                summary.endpoint_id = self.ep.id
                #print summary.Name

                if self.ep.app.model:
                    a = AviarySubmissionUpdate(self.ep.app.model, summary)
                    self.ep.app.update_thread.enqueue(a)

        self.complete = True
