import wallaroo
from work import Work
from datetime import datetime
from mint.update import AviaryUpdate
from requests.exceptions import ConnectionError, HTTPError

import logging
log = logging.getLogger("mint.aviary.sysimage")

class SysimageID(object):
    def __init__(self, name):
        self.resource = "WALLAROO"
        self.subtype = "META"
        self.name = name

class SysimageObj(object):

    types = {"memTotal": int,
             "swapTotal": int,
             "memFree": int,
             "swapFree": int,
             "loadAverage1Min": float,
             "loadAverage5Min": float,
             "loadAverage10Min": float,
             "procTotal": int,
             "procRunning": int}

    def __init__(self, data):
        self.id = SysimageID(data["nodeName"])

        for k,v in data.iteritems():
            if k in self.types:
                setattr(self, k, self.types[k](v))
            else:
                setattr(self, k, v)
            
class QueryWallarooMeta(Work):
    def __init__(self, ep):
        super(QueryWallarooMeta, self).__init__()
        self.ep = ep
        self.connection = None

    def do_the_job(self, foreman):

        # query wallaroo for the node meta data
        # transform it into a WallarooMetaUpdate
        # enqueue it
        if self.connection is None:
            self.connection = wallaroo.client.ConnectionMeta(host=self.ep.url)

        try:
            data = self.connection.fetch_json_resource("/meta/sysinfo/grid9.lab.bos.redhat.com")
        except (ConnectionError, HTTPError), e:
            log.debug("Sysimage: wallaroo client fetch_json_resource failed %s" % str(e))
            data = None

        if data:
            obj = SysimageObj(data)
            update = AviaryUpdate(self.ep.app.model, 
                                  obj,
                                  self.ep.app.model.com_redhat_sesame.Sysimage)
            self.ep.app.update_thread.enqueue(update)


    def completed(self):
        return self.ep.abandoned
