import logging
import os
import random
import sys
import time

from Queue import Queue as ConcurrentQueue, Full, Empty
from crypt import crypt
from datetime import datetime, timedelta
from pprint import pprint
from getpass import getpass
from random import sample
from tempfile import mkstemp
from threading import Thread, Lock, RLock, Condition
from time import clock, sleep
from traceback import print_exc

from parsley.collectionsex import *
from parsley.config import *
from parsley.loggingex import *
from parsley.threadingex import print_threads

log = logging.getLogger("mint.util")

class MintDaemonThread(Thread):
    def __init__(self, app):
        super(MintDaemonThread, self).__init__()

        self.app = app
        self.name = self.__class__.__name__
        self.stop_requested = False

        self.setDaemon(True)
 
    def init(self):
        pass

    def stop(self, timeout=5):
        if self.stop_requested:
            return
        self.stop_requested = True
        if self.isAlive():
            self.join(timeout)
            log.info("%s stopped" % self.getName())

class MintBlockingDaemonThread(MintDaemonThread):
    def __init__(self, app):
        super(MintBlockingDaemonThread, self).__init__(app)

        self._lock = Lock()
        self._condition = Condition(self._lock)

    def stop(self, timeout=5):
        try:
            self._condition.acquire()
            self.stop_requested = True
            self._condition.notify()
        finally:
            self._condition.release()

        if self.isAlive():
            self.join(timeout)
            log.info("%s stopped" % self.getName())

class MintPeriodicProcessThread(MintBlockingDaemonThread):
    def __init__(self, app, interval):
        super(MintPeriodicProcessThread, self).__init__(app)
        
        self.interval = interval

    def run(self):
        while True:
            start = time.time()
            try:
                self.process()
            except:
                log.info("Periodic process failed.")
                log.debug("Periodic process failed", exc_info=True)

            elapsed = time.time() - start
            delta = self.interval - elapsed

            if delta < 0:
                delta = elapsed % self.interval

            then = datetime.now() + timedelta(seconds=delta)
            try:
                self._condition.acquire()
                if not self.stop_requested:
                    log.debug("Periodic process %s sleeping until %s" \
                              % (self.getName(), then.strftime("%H:%M:%S")))
                    self._condition.wait(delta)
                if self.stop_requested:
                    log.debug("Periodic process %s exiting" % self.getName())
                    break
            finally:
                self._condition.release()

    def process(self):
        pass

def prompt_password():
    password = None

    while password is None:
        once = getpass("Enter new password: ")
        twice = getpass("Confirm new password: ")

        if once == twice:
            password = once
        else:
            print "Passwords don't match; try again"

    return password

password_chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"

def crypt_password(password, salt=None):
    if not salt:
        salt = "".join(sample(password_chars, 2))

    return crypt(password, salt)

def ess(num, ending="s"):
    return num != 1 and ending or ""

class ProfilerThread(object):
    def calibrate(self, prof):
        print "Calibrating"
        
        biases = list()
    
        for i in range(3):
            bias = prof.calibrate(50000)
            biases.append(bias)
            print i, bias
        
        prof.bias = sum(biases) / float(3)

        print "Using bias %f" % prof.bias

    def run(self):
        from cProfile import Profile
        from pstats import Stats

        prof = Profile()

        if hasattr(prof, "calibrate"):
            self.calibrate(prof)

        prof.runctx("self.do_run()",
                    globals=globals(),
                    locals=locals())

        fd, path = mkstemp(".profile")

        prof.dump_stats(path)

        stats = Stats(path)

        stats.sort_stats("cumulative").print_stats(15)
        stats.sort_stats("time").print_stats(15)

    def do_run(self):
        raise Exception("Not implemented")
